package com.hzya.frame.plugin.lets.util;

import cn.hutool.core.lang.Assert;
import com.hzya.frame.beanutil.BeanUtil;
import com.hzya.frame.ttxofs.dto.ofssalesordersearch.DetailsDto;
import com.hzya.frame.ttxofs.dto.ofssalesordersearch.HeaderDetailsDto;
import com.hzya.frame.ttxofs.dto.ofssalesordersearch.HeaderDto;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * 处理OFS销售订单，优惠金额分摊逻辑
 *
 * @Author：liuyang
 * @Package：com.hzya.frame.plugin.lets.util
 * @Project：kangarooDataCenterV3
 * @name：AmountAllocationUtil
 * @Date：2024/9/21 18:19
 * @Filename：AmountAllocationUtil
 */
@Component
public class OfsOrderSaleAmountAllocationUtil {

    Logger logger = LoggerFactory.getLogger(OfsOrderSaleAmountAllocationUtil.class);

    /**
     * TOC销售金额批量分摊
     *
     * @author liuyang
     */
    public void batchTocSalesAmountAllocation(List<com.hzya.frame.ttxofs.dto.ofssalesordersearch.HeaderDetailsDto> headerDetailsDtoList) {
        Assert.notNull(headerDetailsDtoList, "headerDetailsDtoList不能为空！");

        headerDetailsDtoList.forEach(headerDetailsDto -> {
            try {
                tocSalesAmountAllocation(headerDetailsDto);
            } catch (Exception e) {
                logger.error("batchTocSalesAmountAllocation方法抛出异常，批量处理TOC销售金额分摊异常", e);
            }
        });
//        logger.info("=记录优惠金额分摊结果开始=");
//        printAmountOfMoneyNum(headerDetailsDtoList);
//        logger.info("=记录优惠金额分摊结果完成=");
    }

    /**
     * TOC销售金额分摊
     *
     * @param headerDetailsDto OFS销售订单对象
     * @author liuyang
     */
    public void tocSalesAmountAllocation(com.hzya.frame.ttxofs.dto.ofssalesordersearch.HeaderDetailsDto headerDetailsDto) throws Exception {
        logger.info("==TOC销售金额分摊开始==");

        //测试
//        HeaderDto header1 = headerDetailsDto.getHeader();
//        header1.setPlatformDiscounts("2.3");
//        header1.setMerchantDiscounts("4.9");
//        header1.setExpertDiscounts("1.2");
//        header1.setPayDiscounts("6.7");

        Assert.notNull(headerDetailsDto, "OFS销售订单对象不能为空");
        HeaderDto header = headerDetailsDto.getHeader();
        List<DetailsDto> detailsOld = headerDetailsDto.getDetails();

        //拷贝一份List集合，并移除没有「商品金额」的OFS销售订单明细行
        List<DetailsDto> details = copyDetailsDto(detailsOld);
        details.removeIf(dto -> dto.getItemTotalAmount() == null || "0".equals(new BigDecimal(dto.getItemTotalAmount()).stripTrailingZeros().toPlainString()));

        //计算分摊百分比
        calculatePercentage(headerDetailsDto, details, header.getCode());

        String platformDiscounts = header.getPlatformDiscounts();
        String merchantDiscounts = header.getMerchantDiscounts();
        String expertDiscounts = header.getExpertDiscounts();
        String payDiscounts = header.getPayDiscounts();

        BigDecimal targetPlatformDiscounts = null;
        BigDecimal targetMerchantDiscounts = null;
        BigDecimal targetExpertDiscounts = null;
        BigDecimal targetPayDiscounts = null;

        //这4个优惠字段，可能没有，也有可能为0！
        if (platformDiscounts != null && !"".equals(platformDiscounts) && !"0".equals(new BigDecimal(platformDiscounts).stripTrailingZeros().toPlainString())) {
            targetPlatformDiscounts = new BigDecimal(platformDiscounts);
        }
        if (merchantDiscounts != null && !"".equals(merchantDiscounts) && !"0".equals(new BigDecimal(merchantDiscounts).stripTrailingZeros().toPlainString())) {
            targetMerchantDiscounts = new BigDecimal(merchantDiscounts);
        }
        if (expertDiscounts != null && !"".equals(expertDiscounts) && !"0".equals(new BigDecimal(expertDiscounts).stripTrailingZeros().toPlainString())) {
            targetExpertDiscounts = new BigDecimal(expertDiscounts);
        }
        if (payDiscounts != null && !"".equals(payDiscounts) && !"0".equals(new BigDecimal(payDiscounts).stripTrailingZeros().toPlainString())) {
            targetPayDiscounts = new BigDecimal(payDiscounts);
        }

        /**
         * OFS销售订单明细行，没有分摊金额的几种情况：
         * 1.对应的OFS销售订单表头某个优惠字段为空或为0
         * 2.明细行没有实付金额或者为0，所以没有分摊比例，不参与分摊
         */
        BigDecimal totalShareTargetPlatformDiscounts = new BigDecimal("0");
        BigDecimal totalShareTargetMerchantDiscounts = new BigDecimal("0");
        BigDecimal totalShareTargetExpertDiscounts = new BigDecimal("0");
        BigDecimal totalShareTargetPayDiscounts = new BigDecimal("0");

        //分摊金额=分摊比例*优惠金额
        for (int i = 0; i < details.size(); i++) {
            DetailsDto detailsDto = details.get(i);
            //分摊比例
            BigDecimal sharingRatio = detailsDto.getSharingRatio();

            //这里应该先8为小数，实际汇总到出库单时，再四舍五入两位小数
            BigDecimal shareTargetPlatformDiscounts = null;
            BigDecimal shareTargetMerchantDiscounts = null;
            BigDecimal shareTargetExpertDiscounts = null;
            BigDecimal shareTargetPayDiscounts = null;
            if (sharingRatio != null && targetPlatformDiscounts != null) {
                shareTargetPlatformDiscounts = targetPlatformDiscounts.multiply(sharingRatio).setScale(2, BigDecimal.ROUND_HALF_UP);
                detailsDto.setOriginalTargetPlatformDiscounts(targetPlatformDiscounts);
                detailsDto.setShareTargetPlatformDiscounts(shareTargetPlatformDiscounts);

                totalShareTargetPlatformDiscounts = totalShareTargetPlatformDiscounts.add(shareTargetPlatformDiscounts);
            }
            if (sharingRatio != null && targetMerchantDiscounts != null) {
                shareTargetMerchantDiscounts = targetMerchantDiscounts.multiply(sharingRatio).setScale(2, BigDecimal.ROUND_HALF_UP);
                detailsDto.setOriginalTargetMerchantDiscounts(targetMerchantDiscounts);
                detailsDto.setShareTargetMerchantDiscounts(shareTargetMerchantDiscounts);

                totalShareTargetMerchantDiscounts = totalShareTargetMerchantDiscounts.add(shareTargetMerchantDiscounts);
            }
            if (sharingRatio != null && targetExpertDiscounts != null) {
                shareTargetExpertDiscounts = targetExpertDiscounts.multiply(sharingRatio).setScale(2, BigDecimal.ROUND_HALF_UP);
                detailsDto.setOriginalTargetExpertDiscounts(targetExpertDiscounts);
                detailsDto.setShareTargetExpertDiscounts(shareTargetExpertDiscounts);

                totalShareTargetExpertDiscounts = totalShareTargetExpertDiscounts.add(shareTargetExpertDiscounts);
            }
            if (sharingRatio != null && targetPayDiscounts != null) {
                shareTargetPayDiscounts = targetPayDiscounts.multiply(sharingRatio).setScale(2, BigDecimal.ROUND_HALF_UP);
                detailsDto.setOriginalTargetPayDiscounts(targetPayDiscounts);
                detailsDto.setShareTargetPayDiscounts(shareTargetPayDiscounts);

                totalShareTargetPayDiscounts = totalShareTargetPayDiscounts.add(shareTargetPayDiscounts);
            }

            //如果是最后一行，则把尾差进行追加
            if (i == details.size() - 1) {
                if (!"0".equals(totalShareTargetPlatformDiscounts.stripTrailingZeros().toPlainString()) && detailsDto.getShareTargetPlatformDiscounts() != null) {
                    //原始优惠金额-sum（分摊金额）
                    BigDecimal subtract = targetPlatformDiscounts.subtract(totalShareTargetPlatformDiscounts);
                    BigDecimal shareTargetPlatformDiscounts1 = detailsDto.getShareTargetPlatformDiscounts();
                    if (subtract.compareTo(BigDecimal.ZERO) > 0) {
                        detailsDto.setShareTargetPlatformDiscounts(shareTargetPlatformDiscounts1.add(subtract));
                    }
                }
                if (!"0".equals(totalShareTargetMerchantDiscounts.stripTrailingZeros().toPlainString()) && detailsDto.getShareTargetMerchantDiscounts() != null) {
                    //原始优惠金额-sum（分摊金额）
                    BigDecimal subtract = targetMerchantDiscounts.subtract(totalShareTargetMerchantDiscounts);
                    BigDecimal shareTargetPlatformDiscounts1 = detailsDto.getShareTargetMerchantDiscounts();
                    if (subtract.compareTo(BigDecimal.ZERO) > 0) {
                        detailsDto.setShareTargetMerchantDiscounts(shareTargetPlatformDiscounts1.add(subtract));
                    }
                }
                if (!"0".equals(totalShareTargetExpertDiscounts.stripTrailingZeros().toPlainString()) && detailsDto.getShareTargetExpertDiscounts() != null) {
                    BigDecimal subtract = targetExpertDiscounts.subtract(totalShareTargetExpertDiscounts);
                    BigDecimal shareTargetExpertDiscounts1 = detailsDto.getShareTargetExpertDiscounts();
                    if (subtract.compareTo(BigDecimal.ZERO) > 0) {
                        detailsDto.setShareTargetExpertDiscounts(shareTargetExpertDiscounts1.add(subtract));
                    }
                }
                if (!"0".equals(totalShareTargetPayDiscounts.stripTrailingZeros().toPlainString()) && detailsDto.getShareTargetPayDiscounts() != null) {
                    BigDecimal subtract = targetPayDiscounts.subtract(totalShareTargetPayDiscounts);
                    BigDecimal shareTargetPayDiscounts1 = detailsDto.getShareTargetPayDiscounts();
                    if (subtract.compareTo(BigDecimal.ZERO) > 0) {
                        detailsDto.setShareTargetPayDiscounts(shareTargetPayDiscounts1.add(subtract));
                    }
                }
            }

            //拷贝回到原始的集合
            copyDetailsDtoOld(details, detailsOld);
        }
        logger.info("==TOC销售金额分摊结束==");
    }

    /**
     * 计算百分比
     * 分摊比例=实付金额/sum(实付金额)
     *
     * @param headerDetailsDto2 OFS销售订单
     * @author liuyang
     */
    private void calculatePercentage(com.hzya.frame.ttxofs.dto.ofssalesordersearch.HeaderDetailsDto headerDetailsDto2, List<DetailsDto> details, String ofsCode) throws Exception {
        Assert.notNull(headerDetailsDto2, "headerDetailsDto2不能为空");
        HeaderDto header1 = headerDetailsDto2.getHeader();
        List<DetailsDto> details1 = headerDetailsDto2.getDetails();

        BigDecimal sumActualPaymentAmountBigDecimal = sumActualPaymentAmount(details1, header1.getCode());
        for (int i = 0; i < details1.size(); i++) {
            DetailsDto detailsDto = details1.get(i);

            String totalPayAmount = detailsDto.getItemTotalAmount();
            //如果实付金额为0则不进行分摊！
            if (totalPayAmount != null && !"0".equals(new BigDecimal(totalPayAmount).stripTrailingZeros().toPlainString())) {
                BigDecimal totalPayAmountBigDecimal = new BigDecimal(totalPayAmount);
                //得到分摊比例
                BigDecimal percentageBigDecimal = totalPayAmountBigDecimal.divide(sumActualPaymentAmountBigDecimal, 20, BigDecimal.ROUND_HALF_UP).setScale(8, BigDecimal.ROUND_HALF_UP);
                if (details != null && details.size() > 0) {
                    for (int j = 0; j < details.size(); j++) {
                        DetailsDto detailsDto1 = details.get(j);
                        if (detailsDto1.getId() != null && detailsDto.getId() != null) {
                            if (detailsDto1.getId().equals(detailsDto.getId())) {
                                details.get(j).setSharingRatio(percentageBigDecimal);
                            }
                        }
                    }
                }
            } else {
                logger.info(totalPayAmount, "totalPayAmount(商品金额)为空！OFS销售订单号：{} 存货编码：{}", ofsCode, detailsDto.getSkuCode());
            }
        }
    }

    /**
     * 累计这一单对应的实付金额
     *
     * @param details1 OFS销售订单明细行
     * @param code     OFS销售订单号
     * @author liuyang
     */
    private BigDecimal sumActualPaymentAmount(List<DetailsDto> details1, String code) throws Exception {
        Assert.notNull(details1, "details1不能为空");
        Assert.notNull(code, "code不能为空");

        BigDecimal actualPaymentAmountBigDecimal = new BigDecimal("0");
        for (int i = 0; i < details1.size(); i++) {
            DetailsDto detailsDto = details1.get(i);

            String itemTotalAmount = detailsDto.getItemTotalAmount();
//            if (itemTotalAmount == null || "".equals(itemTotalAmount.trim())) {
//                Assert.state(false, "销售订单收付金额不能为空 OFS销售订单号：{}", code);
//            }
            if (itemTotalAmount != null && !"0".equals(new BigDecimal(itemTotalAmount).stripTrailingZeros().toPlainString())) {
                BigDecimal itemTotalAmountBigDecimal = new BigDecimal(itemTotalAmount);
                actualPaymentAmountBigDecimal = actualPaymentAmountBigDecimal.add(itemTotalAmountBigDecimal);
                logger.info("商品金额：{}", itemTotalAmount);
            }
        }
        //如果整单所有的商品金额为0，则全部都不参与分摊
//        if ("0".equals(actualPaymentAmountBigDecimal.stripTrailingZeros().toPlainString())) {
//            Assert.state(false, "销售订单收付金额(合并后)不能为0 OFS销售订单号：{}", code);
//        }
        if ("0".equals(actualPaymentAmountBigDecimal.stripTrailingZeros().toPlainString())) {
            logger.info("OFS销售订单号：{} 整单明细行全部的商品金额都为0，都不参与分摊！", code);
        }
        return actualPaymentAmountBigDecimal;
    }

    /**
     * 集合复制
     *
     * @param details 旧OFS销售订单明细行
     * @author liuyang
     */
    private List<DetailsDto> copyDetailsDto(List<DetailsDto> details) throws Exception {
        List<DetailsDto> newDetails = new ArrayList<>();
        for (int i = 0; i < details.size(); i++) {
            DetailsDto detailsDto = details.get(i);

            DetailsDto newDetailsDto1 = new DetailsDto();
            BeanUtil.copyPropertiesV2(detailsDto, newDetailsDto1);
            newDetails.add(newDetailsDto1);
        }
        return newDetails;
    }

    /**
     * 拷贝到原集合中
     *
     * @param newDetails 新OFS销售订单明细行
     * @param detailsOld 旧OFS销售订单明细行
     * @author liuyang
     */
    private void copyDetailsDtoOld(List<DetailsDto> newDetails, List<DetailsDto> detailsOld) throws Exception {
        if (newDetails != null && detailsOld != null) {
            for (int i = 0; i < detailsOld.size(); i++) {
                DetailsDto detailsDto = detailsOld.get(i);
                for (int j = 0; j < newDetails.size(); j++) {
                    DetailsDto detailsDto1 = newDetails.get(j);
                    if (detailsDto.getId().equals(detailsDto1.getId())) {
                        //把分摊后的金额拷贝到旧的OFS销售订单集合中
                        BeanUtil.copyPropertiesV2(detailsDto1, detailsDto);
                    }
                }
            }
        } else {
            logger.error("copyDetailsDtoOld方法newDetails或者detailsOld为空！");
        }
    }

    /**
     * 打印金额参数
     *
     * @param headerDetailsDtoList OFS销售订单号
     */
    public void printAmountOfMoneyNum(List<HeaderDetailsDto> headerDetailsDtoList) {
        try {
            if (headerDetailsDtoList != null && headerDetailsDtoList.size() > 0) {
                for (int i = 0; i < headerDetailsDtoList.size(); i++) {
                    HeaderDetailsDto headerDetailsDto = headerDetailsDtoList.get(i);
                    HeaderDto header = headerDetailsDto.getHeader();
                    List<DetailsDto> details = headerDetailsDto.getDetails();

                    logger.info("OFS销售订单号：{} 平台优惠：{} 商家优惠：{} 达人优惠：{} 支付优惠：{}", header.getCode(), header.getPlatformDiscounts(), header.getMerchantDiscounts(), header.getExpertDiscounts(), header.getPayDiscounts());
                    if (details != null && details.size() > 0) {
                        for (int j = 0; j < details.size(); j++) {
                            DetailsDto detailsDto = details.get(j);

                            logger.info("start================================");
                            logger.info("OFS销售订单明细主键：{}", detailsDto.getId());
                            if (detailsDto.getSharingRatio() != null) {
                                logger.info("分摊比例：{}", detailsDto.getSharingRatio().stripTrailingZeros().toPlainString());
                            } else {
                                logger.info("分摊比例：无");
                            }
                            logger.info("商品金额：{}", detailsDto.getItemTotalAmount());
                            if (detailsDto.getShareTargetPlatformDiscounts() != null) {
                                logger.info("分摊平台优惠：{}", detailsDto.getShareTargetPlatformDiscounts().stripTrailingZeros().toPlainString());
                            } else {
                                logger.info("分摊平台优惠：无");
                            }
                            if (detailsDto.getShareTargetMerchantDiscounts() != null) {
                                logger.info("分摊商家优惠：{}", detailsDto.getShareTargetMerchantDiscounts().stripTrailingZeros().toPlainString());
                            } else {
                                logger.info("分摊商家优惠：无");
                            }
                            if (detailsDto.getShareTargetExpertDiscounts() != null) {
                                logger.info("分摊达人优惠：{}", detailsDto.getShareTargetExpertDiscounts().stripTrailingZeros().toPlainString());
                            } else {
                                logger.info("分摊达人优惠：无");
                            }
                            if (detailsDto.getShareTargetPayDiscounts() != null) {
                                logger.info("分摊支付优惠：{}", detailsDto.getShareTargetPayDiscounts().stripTrailingZeros().toPlainString());
                            } else {
                                logger.info("分摊支付优惠：无");
                            }
                            logger.info("end================================");
                        }
                    }
                }
            }
        } catch (Exception e) {
//            e.printStackTrace();
            logger.error("printAmountOfMoneyNum方法抛出异常", e);
        }
    }
}